@@ -12,10 +12,12 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 """A convenience class around email.Message and email.MIMEMultipart."""
 
+from __future__ import absolute_import
+
 from email import (
     Header,
     Message,
@@ -181,7 +183,7 @@ class EmailMessage(object):
     def string_with_encoding(string_):
         """Return a str object together with an encoding.
 
-        :param string_: A str or unicode object.
+        :param string\\_: A str or unicode object.
         :return: A tuple (str, encoding), where encoding is one of 'ascii',
             'utf-8', or '8-bit', in that preferred order.
         """
