@@ -23,6 +23,7 @@ import re
 import select
 import socket
 import sys
+import tempfile
 import time
 
 from bzrlib import (
@@ -181,6 +182,15 @@ class TestRename(tests.TestCaseInTempDir):
         shape = sorted(os.listdir('.'))
         self.assertEquals(['A', 'B'], shape)
 
+    def test_rename_exception(self):
+        try:
+            osutils.rename('nonexistent_path', 'different_nonexistent_path')
+        except OSError, e:
+            self.assertEqual(e.old_filename, 'nonexistent_path')
+            self.assertEqual(e.new_filename, 'different_nonexistent_path')
+            self.assertTrue('nonexistent_path' in e.strerror)
+            self.assertTrue('different_nonexistent_path' in e.strerror)
+
 
 class TestRandChars(tests.TestCase):
 
@@ -427,6 +437,49 @@ class TestDateTime(tests.TestCase):
         self.assertTrue(-eighteen_hours < offset < eighteen_hours)
 
 
+class TestFdatasync(tests.TestCaseInTempDir):
+
+    def do_fdatasync(self):
+        f = tempfile.NamedTemporaryFile()
+        osutils.fdatasync(f.fileno())
+        f.close()
+
+    @staticmethod
+    def raise_eopnotsupp(*args, **kwargs):
+        raise IOError(errno.EOPNOTSUPP, os.strerror(errno.EOPNOTSUPP))
+
+    @staticmethod
+    def raise_enotsup(*args, **kwargs):
+        raise IOError(errno.ENOTSUP, os.strerror(errno.ENOTSUP))
+
+    def test_fdatasync_handles_system_function(self):
+        self.overrideAttr(os, "fdatasync")
+        self.do_fdatasync()
+
+    def test_fdatasync_handles_no_fdatasync_no_fsync(self):
+        self.overrideAttr(os, "fdatasync")
+        self.overrideAttr(os, "fsync")
+        self.do_fdatasync()
+
+    def test_fdatasync_handles_no_EOPNOTSUPP(self):
+        self.overrideAttr(errno, "EOPNOTSUPP")
+        self.do_fdatasync()
+
+    def test_fdatasync_catches_ENOTSUP(self):
+        enotsup = getattr(errno, "ENOTSUP", None)
+        if enotsup is None:
+            raise tests.TestNotApplicable("No ENOTSUP on this platform")
+        self.overrideAttr(os, "fdatasync", self.raise_enotsup)
+        self.do_fdatasync()
+
+    def test_fdatasync_catches_EOPNOTSUPP(self):
+        enotsup = getattr(errno, "EOPNOTSUPP", None)
+        if enotsup is None:
+            raise tests.TestNotApplicable("No EOPNOTSUPP on this platform")
+        self.overrideAttr(os, "fdatasync", self.raise_eopnotsupp)
+        self.do_fdatasync()
+
+
 class TestLinks(tests.TestCaseInTempDir):
 
     def test_dereference_path(self):
@@ -545,7 +598,7 @@ class TestPumpFile(tests.TestCase):
     """Test pumpfile method."""
 
     def setUp(self):
-        tests.TestCase.setUp(self)
+        super(TestPumpFile, self).setUp()
         # create a test datablock
         self.block_size = 512
         pattern = '0123456789ABCDEF'
@@ -819,6 +872,45 @@ class TestSafeFileId(tests.TestCase):
         self.assertEqual(None, osutils.safe_file_id(None))
 
 
+class TestSendAll(tests.TestCase):
+
+    def test_send_with_disconnected_socket(self):
+        class DisconnectedSocket(object):
+            def __init__(self, err):
+                self.err = err
+            def send(self, content):
+                raise self.err
+            def close(self):
+                pass
+        # All of these should be treated as ConnectionReset
+        errs = []
+        for err_cls in (IOError, socket.error):
+            for errnum in osutils._end_of_stream_errors:
+                errs.append(err_cls(errnum))
+        for err in errs:
+            sock = DisconnectedSocket(err)
+            self.assertRaises(errors.ConnectionReset,
+                osutils.send_all, sock, 'some more content')
+
+    def test_send_with_no_progress(self):
+        # See https://bugs.launchpad.net/bzr/+bug/1047309
+        # It seems that paramiko can get into a state where it doesn't error,
+        # but it returns 0 bytes sent for requests over and over again.
+        class NoSendingSocket(object):
+            def __init__(self):
+                self.call_count = 0
+            def send(self, bytes):
+                self.call_count += 1
+                if self.call_count > 100:
+                    # Prevent the test suite from hanging
+                    raise RuntimeError('too many calls')
+                return 0
+        sock = NoSendingSocket()
+        self.assertRaises(errors.ConnectionReset,
+                          osutils.send_all, sock, 'content')
+        self.assertEqual(1, sock.call_count)
+
+
 class TestPosixFuncs(tests.TestCase):
     """Test that the posix version of normpath returns an appropriate path
        when used with 2 leading slashes."""
@@ -1762,7 +1854,7 @@ class TestDirReader(tests.TestCaseInTempDir):
     _native_to_unicode = None
 
     def setUp(self):
-        tests.TestCaseInTempDir.setUp(self)
+        super(TestDirReader, self).setUp()
         self.overrideAttr(osutils,
                           '_selected_dir_reader', self._dir_reader_class())
 
@@ -1976,7 +2068,7 @@ class TestFailedToLoadExtension(tests.TestCase):
 class TestTerminalWidth(tests.TestCase):
 
     def setUp(self):
-        tests.TestCase.setUp(self)
+        super(TestTerminalWidth, self).setUp()
         self._orig_terminal_size_state = osutils._terminal_size_state
         self._orig_first_terminal_size = osutils._first_terminal_size
         self.addCleanup(self.restore_osutils_globals)
@@ -2063,7 +2155,7 @@ class TestCreationOps(tests.TestCaseInTempDir):
     _test_needs_features = [features.chown_feature]
 
     def setUp(self):
-        tests.TestCaseInTempDir.setUp(self)
+        super(TestCreationOps, self).setUp()
         self.overrideAttr(os, 'chown', self._dummy_chown)
 
         # params set by call to _dummy_chown
