@@ -20,10 +20,13 @@ import time
 from bzrlib import (
     errors,
     revision as _mod_revision,
+    symbol_versioning,
     )
 from bzrlib.tests import TestCaseWithTransport
 from bzrlib.revisionspec import (
+    RevisionInfo,
     RevisionSpec,
+    RevisionSpec_dwim,
     RevisionSpec_tag,
     )
 
@@ -116,7 +119,10 @@ class TestRevisionSpecBase(TestRevisionSpec):
         # If wants_revision_history = True, then _match_on should get the
         # branch revision history
         spec = RevisionSpecMatchOnTrap('foo', _internal=True)
-        spec.in_history(self.tree.branch)
+        spec.wants_revision_history = True
+        self.callDeprecated(['RevisionSpec.wants_revision_history was '
+            'deprecated in 2.5 (RevisionSpecMatchOnTrap).'],
+            spec.in_history, self.tree.branch)
 
         self.assertEqual((self.tree.branch, ['r1' ,'r2']),
                          spec.last_call)
@@ -125,7 +131,6 @@ class TestRevisionSpecBase(TestRevisionSpec):
         # If wants_revision_history = False, then _match_on should get None for
         # the branch revision history
         spec = RevisionSpecMatchOnTrap('foo', _internal=True)
-        spec.wants_revision_history = False
         spec.in_history(self.tree.branch)
 
         self.assertEqual((self.tree.branch, None), spec.last_call)
@@ -142,6 +147,17 @@ class TestOddRevisionSpec(TestRevisionSpec):
         self.assertRaises(TypeError, RevisionSpec.from_string, object())
 
 
+class RevisionSpec_bork(RevisionSpec):
+
+    prefix = 'irrelevant:'
+
+    def _match_on(self, branch, revs):
+        if self.spec == "bork":
+            return RevisionInfo.from_revision_id(branch, "r1")
+        else:
+            raise errors.InvalidRevisionSpec(self.spec, branch)
+
+
 class TestRevisionSpec_dwim(TestRevisionSpec):
 
     # Don't need to test revno's explicitly since TRS_revno already
@@ -185,6 +201,23 @@ class TestRevisionSpec_dwim(TestRevisionSpec):
         self.assertInvalid('1.2..1', invalid_as_revision_id=False)
         self.assertInvalid('1.', invalid_as_revision_id=False)
 
+    def test_append_dwim_revspec(self):
+        original_dwim_revspecs = list(RevisionSpec_dwim._possible_revspecs)
+        def reset_dwim_revspecs():
+            RevisionSpec_dwim._possible_revspecs = original_dwim_revspecs
+        self.addCleanup(reset_dwim_revspecs)
+        RevisionSpec_dwim.append_possible_revspec(RevisionSpec_bork)
+        self.assertAsRevisionId('r1', 'bork')
+
+    def test_append_lazy_dwim_revspec(self):
+        original_dwim_revspecs = list(RevisionSpec_dwim._possible_revspecs)
+        def reset_dwim_revspecs():
+            RevisionSpec_dwim._possible_revspecs = original_dwim_revspecs
+        self.addCleanup(reset_dwim_revspecs)
+        RevisionSpec_dwim.append_possible_lazy_revspec(
+            "bzrlib.tests.test_revisionspec", "RevisionSpec_bork")
+        self.assertAsRevisionId('r1', 'bork')
+
 
 class TestRevisionSpec_revno(TestRevisionSpec):
 
@@ -367,8 +400,7 @@ class TestRevisionSpec_revid(TestRevisionSpec):
         """We can get any revision id in the repository"""
         # XXX: This may change in the future, but for now, it is true
         self.tree2.commit('alt third', rev_id='alt_r3')
-        self.tree.branch.repository.fetch(self.tree2.branch.repository,
-                                          revision_id='alt_r3')
+        self.tree.branch.fetch(self.tree2.branch, 'alt_r3')
         self.assertInHistoryIs(None, 'alt_r3', 'revid:alt_r3')
 
     def test_unicode(self):
@@ -445,8 +477,7 @@ class TestRevisionSpec_before(TestRevisionSpec):
     def test_alt_no_parents(self):
         new_tree = self.make_branch_and_tree('new_tree')
         new_tree.commit('first', rev_id='new_r1')
-        self.tree.branch.repository.fetch(new_tree.branch.repository,
-                                          revision_id='new_r1')
+        self.tree.branch.fetch(new_tree.branch, 'new_r1')
         self.assertInHistoryIs(0, 'null:', 'before:revid:new_r1')
 
     def test_as_revision_id(self):
@@ -669,7 +700,7 @@ class TestRevisionSpec_mainline(TestRevisionSpec):
 class TestRevisionSpec_annotate(TestRevisionSpec):
 
     def setUp(self):
-        TestRevisionSpec.setUp(self)
+        super(TestRevisionSpec_annotate, self).setUp()
         self.tree = self.make_branch_and_tree('annotate-tree')
         self.build_tree_contents([('annotate-tree/file1', '1\n')])
         self.tree.add('file1')
